@@ -2,7 +2,6 @@
 // Name:        src/common/appbase.cpp
 // Purpose:     implements wxAppConsoleBase class
 // Author:      Vadim Zeitlin
-// Modified by:
 // Created:     19.06.2003 (extracted from common/appcmn.cpp)
 // Copyright:   (c) 2003 Vadim Zeitlin <vadim@wxwidgets.org>
 // Licence:     wxWindows licence
@@ -44,45 +43,13 @@
 #include "wx/thread.h"
 #include "wx/stdpaths.h"
 
+#include "wx/private/safecall.h"
+
 #if wxUSE_EXCEPTIONS
-    // Do we have a C++ compiler with enough C++11 support for
-    // std::exception_ptr and functions working with it?
-    #if __cplusplus >= 201103L
-        // Any conforming C++11 compiler should have it, but g++ implementation
-        // of exception handling depends on the availability of the atomic int
-        // operations apparently, so all known version of g++ with C++11 support
-        // (4.7..4.9) fail to provide exception_ptr if the symbol below is not
-        // set to 2 (meaning "always available"), which is notably the case for
-        // MinGW-w64 without -march=486 switch, see #16634.
-        #ifdef __GNUC__
-            // This symbol is always defined in the known g++ version, so
-            // assume that if it isn't defined, things changed for the better
-            // and optimistically suppose that exception_ptr is available.
-            #if !defined(__GCC_ATOMIC_INT_LOCK_FREE) \
-                    || __GCC_ATOMIC_INT_LOCK_FREE > 1
-                #define HAS_EXCEPTION_PTR
-            #endif
-        #else
-            #define HAS_EXCEPTION_PTR
-        #endif
-    #elif wxCHECK_VISUALC_VERSION(11)
-        // VC++ supports it since version 10, even though it doesn't define
-        // __cplusplus to C++11 value, but MSVC 2010 doesn't have a way to test
-        // whether exception_ptr is valid, so we'd need to use a separate bool
-        // flag for it if we wanted to make it work. For now just settle for
-        // only using exception_ptr for VC11 and later.
-        #define HAS_EXCEPTION_PTR
-    #endif
+    #include <exception>        // for std::current_exception()
+    #include <utility>          // for std::swap()
 
-    #ifdef HAS_EXCEPTION_PTR
-        #include <exception>        // for std::current_exception()
-        #include <utility>          // for std::swap()
-    #endif
-
-    #if wxUSE_STL
-        #include <exception>
-        #include <typeinfo>
-    #endif
+    #include <typeinfo>
 #endif // wxUSE_EXCEPTIONS
 
 #if !defined(__WINDOWS__)
@@ -95,6 +62,10 @@
     #include "wx/fontmap.h"
 #endif // wxUSE_FONTMAP
 
+#if wxUSE_LOG
+    #include "wx/private/log.h"
+#endif // wxUSE_LOG
+
 #if wxDEBUG_LEVEL
     #if wxUSE_STACKWALKER
         #include "wx/stackwalk.h"
@@ -106,6 +77,8 @@
     #include "wx/recguard.h"
 #endif // wxDEBUG_LEVEL
 
+#include <memory>
+
 // wxABI_VERSION can be defined when compiling applications but it should be
 // left undefined when compiling the library itself, it is then set to its
 // default value in version.h
@@ -129,7 +102,10 @@
                           const wxString& func,
                           const wxString& cond,
                           const wxString& msg,
-                          wxAppTraits *traits = NULL);
+                          wxAppTraits *traits = nullptr);
+
+    // Used to pass the function name from wxDefaultAssertHandler().
+    static wxString gs_assertFunc;
 #endif // wxDEBUG_LEVEL
 
 #ifdef __WXDEBUG__
@@ -141,11 +117,13 @@
 // global vars
 // ----------------------------------------------------------------------------
 
-wxAppConsole *wxAppConsoleBase::ms_appInstance = NULL;
+wxAppConsole *wxAppConsoleBase::ms_appInstance = nullptr;
 
-wxAppInitializerFunction wxAppConsoleBase::ms_appInitFn = NULL;
+wxAppInitializerFunction wxAppConsoleBase::ms_appInitFn = nullptr;
 
-wxSocketManager *wxAppTraitsBase::ms_manager = NULL;
+int wxAppConsoleBase::ms_fatalErrorExitCode = 255;
+
+wxSocketManager *wxAppTraitsBase::ms_manager = nullptr;
 
 WXDLLIMPEXP_DATA_BASE(wxList) wxPendingDelete;
 
@@ -166,20 +144,10 @@ wxDEFINE_TIED_SCOPED_PTR_TYPE(wxEventLoopBase)
 
 wxAppConsoleBase::wxAppConsoleBase()
 {
-    m_traits = NULL;
-    m_mainLoop = NULL;
-    m_bDoPendingEventProcessing = true;
-
     ms_appInstance = reinterpret_cast<wxAppConsole *>(this);
 
 #ifdef __WXDEBUG__
     SetTraceMasks();
-#if wxUSE_UNICODE
-    // In unicode mode the SetTraceMasks call can cause an apptraits to be
-    // created, but since we are still in the constructor the wrong kind will
-    // be created for GUI apps.  Destroy it so it can be created again later.
-    wxDELETE(m_traits);
-#endif
 #endif
 
     wxEvtHandler::AddFilter(this);
@@ -191,7 +159,7 @@ wxAppConsoleBase::~wxAppConsoleBase()
 
     // we're being destroyed and using this object from now on may not work or
     // even crash so don't leave dangling pointers to it
-    ms_appInstance = NULL;
+    ms_appInstance = nullptr;
 
     delete m_traits;
 }
@@ -200,8 +168,28 @@ wxAppConsoleBase::~wxAppConsoleBase()
 // initialization/cleanup
 // ----------------------------------------------------------------------------
 
+void wxAppConsoleBase::WXAppConstructed()
+{
+    wxASSERT_MSG( !m_fullyConstructed, "must be called only once" );
+
+    m_fullyConstructed = true;
+}
+
 bool wxAppConsoleBase::Initialize(int& WXUNUSED(argc), wxChar **WXUNUSED(argv))
 {
+    if ( IsGUI() )
+    {
+        // GUI wxApp code must call WXAppConstructed() at the very end.
+        wxASSERT_MSG( m_fullyConstructed, "Forgot to call WXAppConstructed()?" );
+    }
+    else // console application
+    {
+        // wxAppConsole doesn't call WXAppConstructed() as otherwise it would
+        // be called twice when it's used as base class of a GUI wxApp, so call
+        // it ourselves here.
+        WXAppConstructed();
+    }
+
 #if defined(__WINDOWS__)
     SetErrorMode(SEM_FAILCRITICALERRORS|SEM_NOOPENFILEERRORBOX);
 #endif
@@ -217,7 +205,7 @@ wxString wxAppConsoleBase::GetAppName() const
         if ( argv )
         {
             // the application name is, by default, the name of its executable file
-            wxFileName::SplitPath(argv[0], NULL, &name, NULL);
+            wxFileName::SplitPath(argv[0], nullptr, &name, nullptr);
         }
 #if wxUSE_STDPATHS
         else // fall back to the executable file name, if we can determine it
@@ -225,7 +213,7 @@ wxString wxAppConsoleBase::GetAppName() const
             const wxString pathExe = wxStandardPaths::Get().GetExecutablePath();
             if ( !pathExe.empty() )
             {
-                wxFileName::SplitPath(pathExe, NULL, &name, NULL);
+                wxFileName::SplitPath(pathExe, nullptr, &name, nullptr);
             }
         }
 #endif // wxUSE_STDPATHS
@@ -258,26 +246,12 @@ void wxAppConsoleBase::CleanUp()
 {
 #if wxUSE_CONFIG
     // Delete the global wxConfig object, if any, and reset it.
-    delete wxConfigBase::Set(NULL);
+    delete wxConfigBase::Set(nullptr);
 #endif // wxUSE_CONFIG
 
     wxDELETE(m_mainLoop);
 }
 
-// The error code to return if OnInit() fails: this is a wxApp member variable
-// in 3.3, but this would break ABI in 3.2, so use a global for it instead.
-static int gs_errorExitCode = -1;
-
-void wxAppConsoleBase::SetErrorExitCode(int code)
-{
-    gs_errorExitCode = code;
-}
-
-int wxAppConsoleBase::GetErrorExitCode() const
-{
-    return gs_errorExitCode;
-}
-
 // ----------------------------------------------------------------------------
 // OnXXX() callbacks
 // ----------------------------------------------------------------------------
@@ -337,7 +311,7 @@ int wxAppConsoleBase::OnExit()
 
 void wxAppConsoleBase::Exit()
 {
-    if (m_mainLoop != NULL)
+    if (m_mainLoop != nullptr)
         ExitMainLoop();
     else
         exit(-1);
@@ -354,22 +328,33 @@ wxAppTraits *wxAppConsoleBase::CreateTraits()
 
 wxAppTraits *wxAppConsoleBase::GetTraits()
 {
-    // FIXME-MT: protect this with a CS?
-    if ( !m_traits )
-    {
-        m_traits = CreateTraits();
+    // If we already have valid traits, just return them.
+    if ( m_traits )
+        return m_traits;
 
-        wxASSERT_MSG( m_traits, wxT("wxApp::CreateTraits() failed?") );
-    }
+    // Otherwise, create a new traits object as it would be unexpected (and
+    // backwards incompatible) to return a null pointer from this function.
+    auto* const traits = CreateTraits();
 
-    return m_traits;
+    // But only remember it if we're fully constructed to prevent using wrong
+    // traits object later: if m_fullyConstructed is false, it means that the
+    // object of the user-defined wxApp-derived class hasn't been fully
+    // constructed yet, and so its possibly overridden CreateTraits() wasn't
+    // called above, so make sure we do call it the next time GetTraits() is
+    // called.
+    if ( m_fullyConstructed )
+        m_traits = traits;
+
+    wxASSERT_MSG( traits, wxT("wxApp::CreateTraits() failed?") );
+
+    return traits;
 }
 
 /* static */
 wxAppTraits *wxAppConsoleBase::GetTraitsIfExists()
 {
     wxAppConsole * const app = GetInstance();
-    return app ? app->GetTraits() : NULL;
+    return app ? app->GetTraits() : nullptr;
 }
 
 /* static */
@@ -429,7 +414,7 @@ bool wxAppConsoleBase::Yield(bool onlyIfNeeded)
     if ( loop )
        return loop->Yield(onlyIfNeeded);
 
-    wxScopedPtr<wxEventLoopBase> tmpLoop(CreateMainLoop());
+    std::unique_ptr<wxEventLoopBase> tmpLoop(CreateMainLoop());
     return tmpLoop->Yield(onlyIfNeeded);
 }
 
@@ -467,7 +452,7 @@ bool wxAppConsoleBase::UsesEventLoop() const
 {
     // in console applications we don't know whether we're going to have an
     // event loop so assume we won't -- unless we already have one running
-    return wxEventLoopBase::GetActive() != NULL;
+    return wxEventLoopBase::GetActive() != nullptr;
 }
 
 // ----------------------------------------------------------------------------
@@ -479,7 +464,7 @@ bool wxAppConsoleBase::IsMainLoopRunning()
 {
     const wxAppConsole * const app = GetInstance();
 
-    return app && app->m_mainLoop != NULL;
+    return app && app->m_mainLoop != nullptr;
 }
 
 int wxAppConsoleBase::FilterEvent(wxEvent& WXUNUSED(event))
@@ -701,7 +686,6 @@ void wxAppConsoleBase::OnUnhandledException()
     {
         throw;
     }
-#if wxUSE_STL
     catch ( std::exception& e )
     {
 #ifdef wxNO_RTTI
@@ -711,7 +695,6 @@ void wxAppConsoleBase::OnUnhandledException()
                     typeid(e).name(), e.what());
 #endif
     }
-#endif // wxUSE_STL
     catch ( ... )
     {
         what = "unknown exception";
@@ -724,6 +707,25 @@ void wxAppConsoleBase::OnUnhandledException()
     );
 }
 
+/* static */
+void wxAppConsoleBase::CallOnUnhandledException()
+{
+    if ( wxTheApp )
+    {
+        wxSafeCall<void>([]()
+        {
+            wxTheApp->OnUnhandledException();
+        }, []()
+        {
+            // And OnUnhandledException() absolutely shouldn't throw,
+            // but we still must account for the possibility that it
+            // did. At least show some information about the exception
+            // in this case by calling our, non-overridden version.
+            wxTheApp->wxAppConsoleBase::OnUnhandledException();
+        });
+    }
+}
+
 // ----------------------------------------------------------------------------
 // exceptions support
 // ----------------------------------------------------------------------------
@@ -733,7 +735,6 @@ bool wxAppConsoleBase::OnExceptionInMainLoop()
     throw;
 }
 
-#ifdef HAS_EXCEPTION_PTR
 static std::exception_ptr gs_storedException;
 
 bool wxAppConsoleBase::StoreCurrentException()
@@ -763,19 +764,6 @@ void wxAppConsoleBase::RethrowStoredException()
     }
 }
 
-#else // !HAS_EXCEPTION_PTR
-
-bool wxAppConsoleBase::StoreCurrentException()
-{
-    return false;
-}
-
-void wxAppConsoleBase::RethrowStoredException()
-{
-}
-
-#endif // HAS_EXCEPTION_PTR/!HAS_EXCEPTION_PTR
-
 #endif // wxUSE_EXCEPTIONS
 
 // ----------------------------------------------------------------------------
@@ -803,7 +791,7 @@ void wxAppConsoleBase::OnInitCmdLine(wxCmdLineParser& parser)
 #if wxUSE_LOG
         {
             wxCMD_LINE_SWITCH,
-            NULL,
+            nullptr,
             OPTION_VERBOSE,
             gettext_noop("generate verbose log messages"),
             wxCMD_LINE_VAL_NONE,
@@ -902,7 +890,11 @@ void wxAppConsoleBase::OnAssert(const wxChar *file,
                                 const wxChar *cond,
                                 const wxChar *msg)
 {
-    OnAssertFailure(file, line, NULL, cond, msg);
+#if wxDEBUG_LEVEL
+    OnAssertFailure(file, line, gs_assertFunc.wc_str(), cond, msg);
+#else
+    OnAssertFailure(file, line, nullptr, cond, msg);
+#endif
 }
 
 // ----------------------------------------------------------------------------
@@ -917,9 +909,6 @@ void wxAppConsoleBase::SetCLocale()
     wxSetlocale(LC_ALL, "");
 }
 
-void* wxAppConsoleBase::WXReservedApp1(void*) { return NULL; }
-void* wxAppConsoleBase::WXReservedApp2(void*) { return NULL; }
-
 // ============================================================================
 // other classes implementations
 // ============================================================================
@@ -932,14 +921,14 @@ void* wxAppConsoleBase::WXReservedApp2(void*) { return NULL; }
 
 wxLog *wxConsoleAppTraitsBase::CreateLogTarget()
 {
-    return new wxLogStderr;
+    return new wxLogOutputBest;
 }
 
 #endif // wxUSE_LOG
 
 wxMessageOutput *wxConsoleAppTraitsBase::CreateMessageOutput()
 {
-    return new wxMessageOutputStderr;
+    return new wxMessageOutputBest;
 }
 
 #if wxUSE_FONTMAP
@@ -954,7 +943,7 @@ wxFontMapper *wxConsoleAppTraitsBase::CreateFontMapper()
 wxRendererNative *wxConsoleAppTraitsBase::CreateRenderer()
 {
     // console applications don't use renderers
-    return NULL;
+    return nullptr;
 }
 
 bool wxConsoleAppTraitsBase::ShowAssertDialog(const wxString& msg)
@@ -1040,7 +1029,7 @@ wxString wxAppTraitsBase::GetAssertStackTrace()
 #if !defined(__WINDOWS__)
     // on Unix stack frame generation may take some time, depending on the
     // size of the executable mainly... warn the user that we are working
-    wxFprintf(stderr, "Collecting stack trace information, please wait...");
+    wxFputs("Collecting stack trace information, please wait...", stderr);
     fflush(stderr);
 #endif // !__WINDOWS__
 
@@ -1053,7 +1042,7 @@ wxString wxAppTraitsBase::GetAssertStackTrace()
         const wxString& GetStackTrace() const { return m_stackTrace; }
 
     protected:
-        virtual void OnStackFrame(const wxStackFrame& frame) wxOVERRIDE
+        virtual void OnStackFrame(const wxStackFrame& frame) override
         {
             // don't show more than maxLines or we could get a dialog too tall
             // to be shown on screen: 20 should be ok everywhere as even with
@@ -1078,19 +1067,18 @@ wxString wxAppTraitsBase::GetAssertStackTrace()
 
             if ( !name.empty() )
             {
-                m_stackTrace << wxString::Format(wxT("%-40s"), name.c_str());
+                m_stackTrace << wxString::Format("%-80s", name);
             }
             else
             {
-                m_stackTrace << wxString::Format(wxT("%p"), frame.GetAddress());
+                m_stackTrace << wxString::Format("%-80p", frame.GetAddress());
             }
 
             if ( frame.HasSourceLocation() )
             {
-                m_stackTrace << wxT('\t')
-                             << frame.GetFileName()
-                             << wxT(':')
-                             << frame.GetLine();
+                m_stackTrace << wxString::Format("%s:%zu",
+                                                 frame.GetFileName(),
+                                                 frame.GetLine());
             }
 
             m_stackTrace << wxT('\n');
@@ -1112,8 +1100,6 @@ wxString wxAppTraitsBase::GetAssertStackTrace()
 }
 #endif // wxUSE_STACKWALKER
 
-void* wxAppTraitsBase::WXReservedAppTraits1(void*) { return NULL; }
-void* wxAppTraitsBase::WXReservedAppTraits2(void*) { return NULL; }
 
 // ============================================================================
 // global functions implementation
@@ -1180,10 +1166,6 @@ wxDefaultAssertHandler(const wxString& file,
                        const wxString& cond,
                        const wxString& msg)
 {
-    // If this option is set, we should abort immediately when assert happens.
-    if ( wxSystemOptions::GetOptionInt("exit-on-assert") )
-        wxAbort();
-
     // FIXME MT-unsafe
     static int s_bInAssert = 0;
 
@@ -1196,6 +1178,10 @@ wxDefaultAssertHandler(const wxString& file,
         return;
     }
 
+    // If this option is set, we should abort immediately when assert happens.
+    if ( wxSystemOptions::GetOptionInt("exit-on-assert") )
+        wxAbort();
+
     if ( !wxTheApp )
     {
         // by default, show the assert dialog box -- we can't customize this
@@ -1205,9 +1191,11 @@ wxDefaultAssertHandler(const wxString& file,
     else
     {
         // let the app process it as it wants
-        // FIXME-UTF8: use wc_str(), not c_str(), when ANSI build is removed
-        wxTheApp->OnAssertFailure(file.c_str(), line, func.c_str(),
-                                  cond.c_str(), msg.c_str());
+
+        // for compatibility, call the old function after stashing the function
+        // name into a global, so that it could pass it to the new one
+        gs_assertFunc = func;
+        wxTheApp->OnAssert(file.wc_str(), line, cond.wc_str(), msg.wc_str());
     }
 }
 
@@ -1235,21 +1223,6 @@ void wxOnAssert(const wxString& file,
     wxTheAssertHandler(file, line, func, cond, wxString());
 }
 
-void wxOnAssert(const wxChar *file,
-                int line,
-                const char *func,
-                const wxChar *cond,
-                const wxChar *msg)
-{
-    // this is the backwards-compatible version (unless we don't use Unicode)
-    // so it could be called directly from the user code and this might happen
-    // even when wxTheAssertHandler is NULL
-#if wxUSE_UNICODE
-    if ( wxTheAssertHandler )
-#endif // wxUSE_UNICODE
-        wxTheAssertHandler(file, line, func, cond, msg);
-}
-
 void wxOnAssert(const char *file,
                 int line,
                 const char *func,
@@ -1268,7 +1241,6 @@ void wxOnAssert(const char *file,
     wxTheAssertHandler(file, line, func, cond, msg);
 }
 
-#if wxUSE_UNICODE
 void wxOnAssert(const char *file,
                 int line,
                 const char *func,
@@ -1294,7 +1266,6 @@ void wxOnAssert(const char *file,
 {
     wxTheAssertHandler(file, line, func, cond, msg);
 }
-#endif // wxUSE_UNICODE
 
 #endif // wxDEBUG_LEVEL
 
@@ -1314,6 +1285,10 @@ static void LINKAGEMODE SetTraceMasks()
         while ( tkn.HasMoreTokens() )
             wxLog::AddTraceMask(tkn.GetNextToken());
     }
+
+    wxString ts;
+    if ( wxGetEnv("WXLOG_TIME_FORMAT", &ts) )
+        wxLog::SetTimestamp(ts);
 #endif // wxUSE_LOG
 }
 
@@ -1337,7 +1312,7 @@ bool DoShowAssertDialog(const wxString& msg)
               wxT("You can also choose [Cancel] to suppress ")
               wxT("further warnings.");
 
-    switch ( ::MessageBox(NULL, msgDlg.t_str(), wxT("wxWidgets Debug Alert"),
+    switch ( ::MessageBox(nullptr, msgDlg.t_str(), wxT("wxWidgets Debug Alert"),
                           MB_YESNOCANCEL | MB_DEFBUTTON2 | MB_ICONSTOP ) )
     {
         case IDYES:
