@@ -2,7 +2,6 @@
 // Name:        src/common/datetimefmt.cpp
 // Purpose:     wxDateTime formatting & parsing code
 // Author:      Vadim Zeitlin
-// Modified by:
 // Created:     11.05.99
 // Copyright:   (c) 1999 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 //              parts of code taken from sndcal library by Scott E. Lee:
@@ -95,7 +94,7 @@ bool GetNumericToken(size_t len,
                      wxString::const_iterator& p,
                      const wxString::const_iterator& end,
                      unsigned long *number,
-                     size_t *numScannedDigits = NULL)
+                     size_t *numScannedDigits = nullptr)
 {
     size_t n = 1;
     wxString s;
@@ -316,17 +315,19 @@ ParseFormatAt(wxString::const_iterator& p,
 wxString wxDateTime::Format(const wxString& formatp, const TimeZone& tz) const
 {
     wxCHECK_MSG( !formatp.empty(), wxEmptyString,
-                 wxT("NULL format in wxDateTime::Format") );
+                 wxT("null format in wxDateTime::Format") );
 
     wxString format = formatp;
 #ifdef __WXOSX__
+#if wxUSE_INTL
     if ( format.Contains("%c") )
         format.Replace("%c", wxUILocale::GetCurrent().GetInfo(wxLOCALE_DATE_TIME_FMT));
     if ( format.Contains("%x") )
         format.Replace("%x", wxUILocale::GetCurrent().GetInfo(wxLOCALE_SHORT_DATE_FMT));
     if ( format.Contains("%X") )
         format.Replace("%X", wxUILocale::GetCurrent().GetInfo(wxLOCALE_TIME_FMT));
-#endif
+#endif // wxUSE_INTL
+#endif // __WXOSX__
     // we have to use our own implementation if the date is out of range of
     // strftime()
 #ifdef wxHAS_STRFTIME
@@ -336,8 +337,8 @@ wxString wxDateTime::Format(const wxString& formatp, const TimeZone& tz) const
     bool isPercent = false;
 
     // We also can't use strftime() if we use non standard specifier: either
-    // our own extension "%l" or one of C99/POSIX specifiers not always
-    // supported under Windows.
+    // our own extension "%l" or one of C99/POSIX specifiers not supported when
+    // using MinGW, see https://sourceforge.net/p/mingw-w64/bugs/793/
     for ( wxString::const_iterator p = format.begin();
           canUseStrftime && p != format.end();
           ++p )
@@ -353,13 +354,13 @@ wxString wxDateTime::Format(const wxString& formatp, const TimeZone& tz) const
         switch ( (*p).GetValue() )
         {
             case 'l':
-#ifdef __WINDOWS__
+#ifdef __MINGW32__
             case 'F':
             case 'g':
             case 'G':
             case 'V':
             case 'z':
-#endif // __WINDOWS__
+#endif // __MINGW32__
                 canUseStrftime = false;
                 break;
         }
@@ -393,7 +394,7 @@ wxString wxDateTime::Format(const wxString& formatp, const TimeZone& tz) const
     tmTimeOnly.tm_year = 76;
     tmTimeOnly.tm_isdst = 0;        // no DST, we adjust for tz ourselves
 
-    wxString tmp, res, fmt;
+    wxString res, fmt;
     for ( wxString::const_iterator p = format.begin(); p != format.end(); ++p )
     {
         if ( *p != wxT('%') )
@@ -868,7 +869,7 @@ wxDateTime::ParseRfc822Date(const wxString& originalDate, wxString::const_iterat
     // 1. week day (optional)
     // if there is a week day present, it must be separated
     // by a comma at position [3].
-    if ( date.Length() > 3 && date[3] == ',' )
+    if ( date.length() > 3 && date[3] == ',' )
     {
         const wxDateTime::WeekDay
             wd = GetWeekDayFromName(p, pEnd, Name_Abbr, DateLang_English);
@@ -1019,7 +1020,7 @@ const char* wxDateTime::ParseRfc822Date(const char* date)
     wxString::const_iterator end;
     wxString dateStr(date);
     if ( !ParseRfc822Date(dateStr, &end) )
-        return NULL;
+        return nullptr;
 
     return date + dateStr.IterOffsetInMBStr(end);
 }
@@ -1029,7 +1030,7 @@ const wchar_t* wxDateTime::ParseRfc822Date(const wchar_t* date)
     wxString::const_iterator end;
     wxString dateStr(date);
     if ( !ParseRfc822Date(dateStr, &end) )
-        return NULL;
+        return nullptr;
 
     return date + (end - dateStr.begin());
 }
@@ -1043,7 +1044,6 @@ wxDateTime::ParseFormat(const wxString& date,
     wxCHECK_MSG( !format.empty(), false, "format can't be empty" );
     wxCHECK_MSG( endParse, false, "end iterator pointer must be specified" );
 
-    wxString str;
     unsigned long num;
 
     // what fields have we found?
@@ -1105,13 +1105,20 @@ wxDateTime::ParseFormat(const wxString& date,
         }
 
         // start of a format specification
+        ++fmt;
+
+        // skip the optional character specifying the padding: this is a GNU
+        // extension which we need to support as this is used in the default
+        // date formats for some locales (but luckily this is simple to do)
+        if ( *fmt == '-' || *fmt == '_' || *fmt == '0' )
+            ++fmt;
 
         // parse the optional width
         size_t width = 0;
-        while ( wxIsdigit(*++fmt) )
+        while ( wxIsdigit(*fmt) )
         {
             width *= 10;
-            width += *fmt - '0';
+            width += *fmt++ - '0';
         }
 
         // the default widths for the various fields
@@ -1549,9 +1556,7 @@ wxDateTime::ParseFormat(const wxString& date,
                     else if
                     (
                         *input == wxS('-')
-#if wxUSE_UNICODE
                         || *input == wxString::FromUTF8("\xe2\x88\x92")
-#endif
                     )
                         minusFound = true;
                     else
@@ -1736,7 +1741,7 @@ wxDateTime::ParseFormat(const char* date,
     wxString::const_iterator end;
     wxString dateStr(date);
     if ( !ParseFormat(dateStr, format, dateDef, &end) )
-        return NULL;
+        return nullptr;
 
     return date + dateStr.IterOffsetInMBStr(end);
 }
@@ -1749,7 +1754,7 @@ wxDateTime::ParseFormat(const wchar_t* date,
     wxString::const_iterator end;
     wxString dateStr(date);
     if ( !ParseFormat(dateStr, format, dateDef, &end) )
-        return NULL;
+        return nullptr;
 
     return date + (end - dateStr.begin());
 }
@@ -1829,7 +1834,7 @@ const char* wxDateTime::ParseDateTime(const char* date)
     wxString::const_iterator end;
     wxString dateStr(date);
     if ( !ParseDateTime(dateStr, &end) )
-        return NULL;
+        return nullptr;
 
     return date + dateStr.IterOffsetInMBStr(end);
 }
@@ -1839,7 +1844,7 @@ const wchar_t* wxDateTime::ParseDateTime(const wchar_t* date)
     wxString::const_iterator end;
     wxString dateStr(date);
     if ( !ParseDateTime(dateStr, &end) )
-        return NULL;
+        return nullptr;
 
     return date + (end - dateStr.begin());
 }
@@ -2213,7 +2218,7 @@ const char* wxDateTime::ParseDate(const char* date)
     wxString::const_iterator end;
     wxString dateStr(date);
     if ( !ParseDate(dateStr, &end) )
-        return NULL;
+        return nullptr;
 
     return date + dateStr.IterOffsetInMBStr(end);
 }
@@ -2223,7 +2228,7 @@ const wchar_t* wxDateTime::ParseDate(const wchar_t* date)
     wxString::const_iterator end;
     wxString dateStr(date);
     if ( !ParseDate(dateStr, &end) )
-        return NULL;
+        return nullptr;
 
     return date + (end - dateStr.begin());
 }
@@ -2289,7 +2294,7 @@ const char* wxDateTime::ParseTime(const char* date)
     wxString::const_iterator end;
     wxString dateStr(date);
     if ( !ParseTime(dateStr, &end) )
-        return NULL;
+        return nullptr;
 
     return date + dateStr.IterOffsetInMBStr(end);
 }
@@ -2299,7 +2304,7 @@ const wchar_t* wxDateTime::ParseTime(const wchar_t* date)
     wxString::const_iterator end;
     wxString dateStr(date);
     if ( !ParseTime(dateStr, &end) )
-        return NULL;
+        return nullptr;
 
     return date + (end - dateStr.begin());
 }
@@ -2375,7 +2380,7 @@ wxString wxTimeSpan::Format(const wxString& format) const
     }
 
     wxCHECK_MSG( !format.empty(), str,
-                 wxT("NULL format in wxTimeSpan::Format") );
+                 wxT("null format in wxTimeSpan::Format") );
 
     str.Alloc(format.length());
 
@@ -2397,10 +2402,16 @@ wxString wxTimeSpan::Format(const wxString& format) const
 
     for ( wxString::const_iterator pch = format.begin(); pch != format.end(); ++pch )
     {
-        wxChar ch = *pch;
+        wxUniChar ch = *pch;
 
         if ( ch == wxT('%') )
         {
+            if ( ++pch == format.end() )
+            {
+                wxFAIL_MSG( wxT("trailing '%' in format string") );
+                break;
+            }
+
             // the start of the format specification of the printf() below
             wxString fmtPrefix(wxT('%'));
 
@@ -2410,8 +2421,8 @@ wxString wxTimeSpan::Format(const wxString& format) const
             // the number of digits for the format string, 0 if unused
             unsigned digits = 0;
 
-            ch = *++pch;    // get the format spec char
-            switch ( ch )
+            ch = *pch;    // get the format spec char
+            switch ( ch.GetValue() )
             {
                 default:
                     wxFAIL_MSG( wxT("invalid format character") );
