@@ -2,7 +2,6 @@
 // Name:        src/common/menucmn.cpp
 // Purpose:     wxMenu and wxMenuBar methods common to all ports
 // Author:      Vadim Zeitlin
-// Modified by:
 // Created:     26.10.99
 // Copyright:   (c) wxWidgets team
 // Licence:     wxWindows licence
@@ -144,7 +143,7 @@ wxCONSTRUCTOR_DUMMY( wxMenuBar )
 const wxMenuInfoHelperList& wxMenuBarBase::GetMenuInfos() const
 {
     wxMenuInfoHelperList* list = const_cast< wxMenuInfoHelperList* > (& m_menuInfos);
-    WX_CLEAR_LIST( wxMenuInfoHelperList, *list);
+    wxClearList(*list);
     for (size_t i = 0 ; i < GetMenuCount(); ++i)
     {
         wxMenuInfoHelper* info = new wxMenuInfoHelper();
@@ -259,7 +258,7 @@ wxMenuItemBase::wxMenuItemBase(wxMenu *parentMenu,
             m_id = itemid;
     }
 
-    // notice that parentMenu can be NULL: the item can be attached to the menu
+    // notice that parentMenu can be null: the item can be attached to the menu
     // later with SetMenu()
 
     m_parentMenu  = parentMenu;
@@ -337,13 +336,6 @@ wxString wxMenuItemBase::GetLabelText(const wxString& text)
     return wxStripMenuCodes(text, wxStrip_Menu);
 }
 
-#if WXWIN_COMPATIBILITY_2_8
-wxString wxMenuItemBase::GetLabelFromText(const wxString& text)
-{
-    return GetLabelText(text);
-}
-#endif
-
 void wxMenuItemBase::SetBitmap(const wxBitmapBundle& bmp)
 {
     m_bitmap = bmp;
@@ -379,18 +371,18 @@ bool wxMenuBase::ms_locked = true;
 
 void wxMenuBase::Init(long style)
 {
-    m_menuBar = NULL;
-    m_menuParent = NULL;
+    m_menuBar = nullptr;
+    m_menuParent = nullptr;
 
-    m_invokingWindow = NULL;
+    m_invokingWindow = nullptr;
     m_style = style;
-    m_clientData = NULL;
+    m_clientData = nullptr;
     m_eventHandler = this;
 }
 
 wxMenuBase::~wxMenuBase()
 {
-    WX_CLEAR_LIST(wxMenuItemList, m_items);
+    wxClearList(m_items);
 }
 
 // ----------------------------------------------------------------------------
@@ -399,14 +391,14 @@ wxMenuBase::~wxMenuBase()
 
 void wxMenuBase::AddSubMenu(wxMenu *submenu)
 {
-    wxCHECK_RET( submenu, wxT("can't add a NULL submenu") );
+    wxCHECK_RET( submenu, wxT("can't add a null submenu") );
 
     submenu->SetParent((wxMenu *)this);
 }
 
 wxMenuItem* wxMenuBase::DoAppend(wxMenuItem *item)
 {
-    wxCHECK_MSG( item, NULL, wxT("invalid item in wxMenu::Append()") );
+    wxCHECK_MSG( item, nullptr, wxT("invalid item in wxMenu::Append()") );
 
     m_items.Append(item);
     item->SetMenu((wxMenu*)this);
@@ -420,7 +412,7 @@ wxMenuItem* wxMenuBase::DoAppend(wxMenuItem *item)
 
 wxMenuItem* wxMenuBase::Insert(size_t pos, wxMenuItem *item)
 {
-    wxCHECK_MSG( item, NULL, wxT("invalid item in wxMenu::Insert") );
+    wxCHECK_MSG( item, nullptr, wxT("invalid item in wxMenu::Insert") );
 
     if ( pos == GetMenuItemCount() )
     {
@@ -428,7 +420,7 @@ wxMenuItem* wxMenuBase::Insert(size_t pos, wxMenuItem *item)
     }
     else
     {
-        wxCHECK_MSG( pos < GetMenuItemCount(), NULL,
+        wxCHECK_MSG( pos < GetMenuItemCount(), nullptr,
                      wxT("invalid index in wxMenu::Insert") );
 
         return DoInsert(pos, item);
@@ -437,10 +429,10 @@ wxMenuItem* wxMenuBase::Insert(size_t pos, wxMenuItem *item)
 
 wxMenuItem* wxMenuBase::DoInsert(size_t pos, wxMenuItem *item)
 {
-    wxCHECK_MSG( item, NULL, wxT("invalid item in wxMenu::Insert()") );
+    wxCHECK_MSG( item, nullptr, wxT("invalid item in wxMenu::Insert()") );
 
     wxMenuItemList::compatibility_iterator node = m_items.Item(pos);
-    wxCHECK_MSG( node, NULL, wxT("invalid index in wxMenu::Insert()") );
+    wxCHECK_MSG( node, nullptr, wxT("invalid index in wxMenu::Insert()") );
 
     m_items.Insert(node, item);
     item->SetMenu((wxMenu*)this);
@@ -454,12 +446,12 @@ wxMenuItem* wxMenuBase::DoInsert(size_t pos, wxMenuItem *item)
 
 wxMenuItem *wxMenuBase::Remove(wxMenuItem *item)
 {
-    wxCHECK_MSG( item, NULL, wxT("invalid item in wxMenu::Remove") );
+    wxCHECK_MSG( item, nullptr, wxT("invalid item in wxMenu::Remove") );
 
     wxMenuItemList::compatibility_iterator node = m_items.Find(item);
 
     // if we get here, the item is valid or one of Remove() functions is broken
-    wxCHECK_MSG( node, NULL, wxT("removing item not in the menu?") );
+    wxCHECK_MSG( node, nullptr, wxT("removing item not in the menu?") );
 
     // call DoRemove() before removing the item from the list of items as the
     // existing code in port-specific implementation may rely on the item still
@@ -475,11 +467,11 @@ wxMenuItem *wxMenuBase::Remove(wxMenuItem *item)
 
 wxMenuItem *wxMenuBase::DoRemove(wxMenuItem *item)
 {
-    item->SetMenu(NULL);
+    item->SetMenu(nullptr);
     wxMenu *submenu = item->GetSubMenu();
     if ( submenu )
     {
-        submenu->SetParent(NULL);
+        submenu->SetParent(nullptr);
 #if wxUSE_MENUBAR
         if ( submenu->IsAttached() )
             submenu->Detach();
@@ -502,7 +494,7 @@ bool wxMenuBase::DoDelete(wxMenuItem *item)
     wxCHECK_MSG( item2, false, wxT("failed to delete menu item") );
 
     // don't delete the submenu
-    item2->SetSubMenu(NULL);
+    item2->SetSubMenu(nullptr);
 
     delete item2;
 
@@ -562,9 +554,9 @@ int wxMenuBase::FindItem(const wxString& text) const
 wxMenuItem *wxMenuBase::FindItem(int itemId, wxMenu **itemMenu) const
 {
     if ( itemMenu )
-        *itemMenu = NULL;
+        *itemMenu = nullptr;
 
-    wxMenuItem *item = NULL;
+    wxMenuItem *item = nullptr;
     for ( wxMenuItemList::compatibility_iterator node = m_items.GetFirst();
           node && !item;
           node = node->GetNext() )
@@ -583,7 +575,7 @@ wxMenuItem *wxMenuBase::FindItem(int itemId, wxMenu **itemMenu) const
         else
         {
             // don't exit the loop
-            item = NULL;
+            item = nullptr;
         }
     }
 
@@ -593,7 +585,7 @@ wxMenuItem *wxMenuBase::FindItem(int itemId, wxMenu **itemMenu) const
 // non recursive search
 wxMenuItem *wxMenuBase::FindChildItem(int itemid, size_t *ppos) const
 {
-    wxMenuItem *item = NULL;
+    wxMenuItem *item = nullptr;
     wxMenuItemList::compatibility_iterator node = GetMenuItems().GetFirst();
 
     size_t pos;
@@ -620,7 +612,7 @@ wxMenuItem *wxMenuBase::FindChildItem(int itemid, size_t *ppos) const
 // find by position
 wxMenuItem* wxMenuBase::FindItemByPosition(size_t position) const
 {
-    wxCHECK_MSG( position < m_items.GetCount(), NULL,
+    wxCHECK_MSG( position < m_items.GetCount(), nullptr,
                  wxT("wxMenu::FindItemByPosition(): invalid menu index") );
 
     return m_items.Item( position )->GetData();
@@ -689,7 +681,7 @@ bool wxMenuBase::DoProcessEvent(wxMenuBase* menu, wxEvent& event, wxWindow* win)
     event.SetEventObject(menu);
 
 #if wxUSE_MENUBAR
-    wxMenuBar* const mb = menu ? menu->GetMenuBar() : NULL;
+    wxMenuBar* const mb = menu ? menu->GetMenuBar() : nullptr;
 #else
     bool mb = false;
 #endif
@@ -769,7 +761,7 @@ wxMenuBar* wxMenuBase::GetMenuBar() const
 void wxMenuBase::Attach(wxMenuBarBase *menubar)
 {
     // use Detach() instead!
-    wxASSERT_MSG( menubar, wxT("menu can't be attached to NULL menubar") );
+    wxASSERT_MSG( menubar, wxT("menu can't be attached to null menubar") );
 
     // use IsAttached() to prevent this from happening
     wxASSERT_MSG( !m_menuBar, wxT("attaching menu twice?") );
@@ -782,7 +774,7 @@ void wxMenuBase::Detach()
     // use IsAttached() to prevent this from happening
     wxASSERT_MSG( m_menuBar, wxT("detaching unattached menu?") );
 
-    m_menuBar = NULL;
+    m_menuBar = nullptr;
 }
 #endif // wxUSE_MENUBAR
 
@@ -803,7 +795,7 @@ void wxMenuBase::SetInvokingWindow(wxWindow *win)
 
 wxWindow *wxMenuBase::GetWindow() const
 {
-    // only the top level menus have non-NULL invoking window or a pointer to
+    // only the top level menus have non-null invoking window or a pointer to
     // the menu bar so recurse upwards until we find it
     const wxMenuBase *menu = this;
     while ( menu->GetParent() )
@@ -904,12 +896,12 @@ wxString wxMenuBase::GetHelpString( int itemid ) const
 wxMenuBarBase::wxMenuBarBase()
 {
     // not attached yet
-    m_menuBarFrame = NULL;
+    m_menuBarFrame = nullptr;
 }
 
 wxMenuBarBase::~wxMenuBarBase()
 {
-    WX_CLEAR_LIST(wxMenuList, m_menus);
+    wxClearList(m_menus);
 }
 
 // ----------------------------------------------------------------------------
@@ -920,14 +912,14 @@ wxMenuBarBase::~wxMenuBarBase()
 wxMenu *wxMenuBarBase::GetMenu(size_t pos) const
 {
     wxMenuList::compatibility_iterator node = m_menus.Item(pos);
-    wxCHECK_MSG( node, NULL, wxT("bad index in wxMenuBar::GetMenu()") );
+    wxCHECK_MSG( node, nullptr, wxT("bad index in wxMenuBar::GetMenu()") );
 
     return node->GetData();
 }
 
 bool wxMenuBarBase::Append(wxMenu *menu, const wxString& title)
 {
-    wxCHECK_MSG( menu, false, wxT("can't append NULL menu") );
+    wxCHECK_MSG( menu, false, wxT("can't append null menu") );
     wxCHECK_MSG( !title.empty(), false, wxT("can't append menu with empty title") );
 
     m_menus.Append(menu);
@@ -945,7 +937,7 @@ bool wxMenuBarBase::Insert(size_t pos, wxMenu *menu,
     }
     else // not at the end
     {
-        wxCHECK_MSG( menu, false, wxT("can't insert NULL menu") );
+        wxCHECK_MSG( menu, false, wxT("can't insert null menu") );
 
         wxMenuList::compatibility_iterator node = m_menus.Item(pos);
         wxCHECK_MSG( node, false, wxT("bad index in wxMenuBar::Insert()") );
@@ -960,10 +952,10 @@ bool wxMenuBarBase::Insert(size_t pos, wxMenu *menu,
 wxMenu *wxMenuBarBase::Replace(size_t pos, wxMenu *menu,
                                const wxString& WXUNUSED(title))
 {
-    wxCHECK_MSG( menu, NULL, wxT("can't insert NULL menu") );
+    wxCHECK_MSG( menu, nullptr, wxT("can't insert null menu") );
 
     wxMenuList::compatibility_iterator node = m_menus.Item(pos);
-    wxCHECK_MSG( node, NULL, wxT("bad index in wxMenuBar::Replace()") );
+    wxCHECK_MSG( node, nullptr, wxT("bad index in wxMenuBar::Replace()") );
 
     wxMenu *menuOld = node->GetData();
     node->SetData(menu);
@@ -977,7 +969,7 @@ wxMenu *wxMenuBarBase::Replace(size_t pos, wxMenu *menu,
 wxMenu *wxMenuBarBase::Remove(size_t pos)
 {
     wxMenuList::compatibility_iterator node = m_menus.Item(pos);
-    wxCHECK_MSG( node, NULL, wxT("bad index in wxMenuBar::Remove()") );
+    wxCHECK_MSG( node, nullptr, wxT("bad index in wxMenuBar::Remove()") );
 
     wxMenu *menu = node->GetData();
     m_menus.Erase(node);
@@ -1022,8 +1014,8 @@ void wxMenuBarBase::Detach()
 {
     wxASSERT_MSG( IsAttached(), wxT("detaching unattached menubar") );
 
-    m_menuBarFrame = NULL;
-    SetParent(NULL);
+    m_menuBarFrame = nullptr;
+    SetParent(nullptr);
 }
 
 // ----------------------------------------------------------------------------
@@ -1033,9 +1025,9 @@ void wxMenuBarBase::Detach()
 wxMenuItem *wxMenuBarBase::FindItem(int itemid, wxMenu **menu) const
 {
     if ( menu )
-        *menu = NULL;
+        *menu = nullptr;
 
-    wxMenuItem *item = NULL;
+    wxMenuItem *item = nullptr;
     size_t count = GetMenuCount(), i;
     wxMenuList::const_iterator it;
     for ( i = 0, it = m_menus.begin(); !item && (i < count); i++, ++it )
@@ -1147,24 +1139,11 @@ void wxMenuBarBase::UpdateMenus()
     {
         wxMenu* menu;
         menu = GetMenu( n );
-        if (menu != NULL)
+        if (menu != nullptr)
             menu->UpdateUI();
     }
 }
 
-#if WXWIN_COMPATIBILITY_2_8
-// get or change the label of the menu at given position
-void wxMenuBarBase::SetLabelTop(size_t pos, const wxString& label)
-{
-    SetMenuLabel(pos, label);
-}
-
-wxString wxMenuBarBase::GetLabelTop(size_t pos) const
-{
-    return GetMenuLabelText(pos);
-}
-#endif
-
 #endif // wxUSE_MENUBAR
 
 #endif // wxUSE_MENUS
