@@ -2,7 +2,6 @@
 // Name:        src/osx/menu_osx.cpp
 // Purpose:     wxMenu, wxMenuBar, wxMenuItem
 // Author:      Stefan Csomor
-// Modified by:
 // Created:     1998-01-01
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
@@ -31,13 +30,14 @@
 #endif
 
 #include "wx/osx/private.h"
-#include "wx/scopedptr.h"
 #include "wx/private/menuradio.h" // for wxMenuRadioItemsData
 
 // other standard headers
 // ----------------------
 #include <string.h>
 
+#include <memory>
+
 wxIMPLEMENT_ABSTRACT_CLASS(wxMenuImpl, wxObject);
 
 wxMenuImpl::~wxMenuImpl()
@@ -47,7 +47,7 @@ wxMenuImpl::~wxMenuImpl()
 // the (popup) menu title has this special menuid
 static const int idMenuTitle = -3;
 
-wxScopedPtr<wxMenu> gs_emptyMenuBar;
+std::unique_ptr<wxMenu> gs_emptyMenuBar;
 
 // ============================================================================
 // implementation
@@ -62,7 +62,7 @@ void wxMenu::Init()
     m_doBreak = false;
     m_allowRearrange = true;
     m_noEventsMode = false;
-    m_radioData = NULL;
+    m_radioData = nullptr;
 
     m_peer = wxMenuImpl::Create( this, wxStripMenuCodes(m_title, wxStrip_Menu) );
 
@@ -86,7 +86,7 @@ WXHMENU wxMenu::GetHMenu() const
 {
     if ( m_peer )
         return m_peer->GetHMenu();
-    return NULL;
+    return nullptr;
 }
 
 void wxMenu::SetAllowRearrange( bool allow )
@@ -108,13 +108,13 @@ bool wxMenu::OSXGetRadioGroupRange(int pos, int *start, int *end) const
 // append a new item or submenu to the menu
 bool wxMenu::DoInsertOrAppend(wxMenuItem *item, size_t pos)
 {
-    wxASSERT_MSG( item != NULL, wxT("can't append NULL item to the menu") );
+    wxASSERT_MSG( item != nullptr, wxT("can't append nullptr item to the menu") );
     GetPeer()->InsertOrAppend( item, pos );
 
     wxMenu *pSubMenu = item->GetSubMenu() ;
-    if ( pSubMenu != NULL )
+    if ( pSubMenu != nullptr )
     {
-        wxASSERT_MSG( pSubMenu->GetHMenu() != NULL , wxT("invalid submenu added"));
+        wxASSERT_MSG( pSubMenu->GetHMenu() != nullptr , wxT("invalid submenu added"));
         pSubMenu->m_menuParent = this ;
 
         pSubMenu->DoRearrange();
@@ -179,7 +179,7 @@ wxMenuItem* wxMenu::DoAppend(wxMenuItem *item)
     if (wxMenuBase::DoAppend(item) && DoInsertOrAppend(item) )
         return item;
 
-    return NULL;
+    return nullptr;
 }
 
 wxMenuItem* wxMenu::DoInsert(size_t pos, wxMenuItem *item)
@@ -187,7 +187,7 @@ wxMenuItem* wxMenu::DoInsert(size_t pos, wxMenuItem *item)
     if (wxMenuBase::DoInsert(pos, item) && DoInsertOrAppend(item, pos))
         return item;
 
-    return NULL;
+    return nullptr;
 }
 
 wxMenuItem *wxMenu::DoRemove(wxMenuItem *item)
@@ -217,7 +217,7 @@ wxMenuItem *wxMenu::DoRemove(wxMenuItem *item)
     }
 
     // DoRemove() (unlike Remove) can only be called for existing item!
-    wxCHECK_MSG( node, NULL, wxT("bug in wxMenu::Remove logic") );
+    wxCHECK_MSG( node, nullptr, wxT("bug in wxMenu::Remove logic") );
 
     wxOSXMenuRemoveItem(m_hMenu , pos );
     */
@@ -265,7 +265,7 @@ void wxMenu::DoRearrange()
     if ( !AllowRearrange() )
         return;
 
-    wxMenuItem* previousItem = NULL ;
+    wxMenuItem* previousItem = nullptr ;
     size_t pos ;
     wxMenuItemList::compatibility_iterator node;
     wxMenuItem *item;
@@ -295,9 +295,9 @@ void wxMenu::DoRearrange()
                 // separator at the menu start or end nor 2 consecutive
                 // separators
                 wxMenuItemList::compatibility_iterator nextNode = node->GetNext();
-                wxMenuItem *next = nextNode ? nextNode->GetData() : NULL;
+                wxMenuItem *next = nextNode ? nextNode->GetData() : nullptr;
 
-                wxMenuItem *sepToHide = 0;
+                wxMenuItem *sepToHide = nullptr;
                 if ( !previousItem && next && next->IsSeparator() )
                 {
                     // next (i.e. second as we must be first) item is
@@ -306,7 +306,7 @@ void wxMenu::DoRearrange()
                     sepToHide = next;
                 }
                 else if ( GetMenuItems().GetCount() == pos + 1 &&
-                            previousItem != NULL &&
+                            previousItem != nullptr &&
                                 previousItem->IsSeparator() )
                 {
                     // prev item is a trailing separator we want to hide
@@ -395,7 +395,7 @@ void wxMenu::HandleMenuItemHighlighted( wxMenuItem* item )
 
 void wxMenu::DoHandleMenuOpenedOrClosed(wxEventType evtType)
 {
-    // Popup menu being currently shown or NULL, defined in wincmn.cpp.
+    // Popup menu being currently shown or nullptr, defined in wincmn.cpp.
     extern wxMenu *wxCurrentPopupMenu;
 
     // Set the id to allow wxMenuEvent::IsPopup() to work correctly.
@@ -473,10 +473,10 @@ auto-merge for MDI in case this will be necessary
 
 */
 
-wxMenuBar* wxMenuBar::s_macInstalledMenuBar = NULL ;
-wxMenuBar* wxMenuBar::s_macCommonMenuBar = NULL ;
+wxMenuBar* wxMenuBar::s_macInstalledMenuBar = nullptr ;
+wxMenuBar* wxMenuBar::s_macCommonMenuBar = nullptr ;
 bool     wxMenuBar::s_macAutoWindowMenu = true ;
-WXHMENU  wxMenuBar::s_macWindowMenuHandle = NULL ;
+WXHMENU  wxMenuBar::s_macWindowMenuHandle = nullptr ;
 
 
 const int firstMenuPos = 1; // to account for the 0th application menu on mac
@@ -549,7 +549,7 @@ void wxMenuBar::Init()
     }
     
     m_eventHandler = this;
-    m_menuBarFrame = NULL;
+    m_menuBarFrame = nullptr;
     m_rootMenu = new wxMenu();
     m_rootMenu->Attach(this);
 
@@ -583,12 +583,12 @@ wxMenuBar::wxMenuBar(size_t count, wxMenu *menus[], const wxString titles[], lon
 wxMenuBar::~wxMenuBar()
 {
     if (s_macCommonMenuBar == this)
-        s_macCommonMenuBar = NULL;
+        s_macCommonMenuBar = nullptr;
 
     MacUninstallMenuBar();
     wxDELETE( m_rootMenu );
     // apple menu is a submenu, therefore we don't have to delete it
-    m_appleMenu = NULL;
+    m_appleMenu = nullptr;
 
     // deleting the root menu also removes all its wxMenu* submenus, therefore
     // we must avoid double deleting them in the superclass destructor
@@ -605,7 +605,7 @@ void wxMenuBar::MacUninstallMenuBar()
   if (s_macInstalledMenuBar == this)
   {
     gs_emptyMenuBar->GetPeer()->MakeRoot();
-    s_macInstalledMenuBar = NULL;
+    s_macInstalledMenuBar = nullptr;
   }
 }
 
@@ -618,15 +618,15 @@ void wxMenuBar::MacInstallMenuBar()
     
     // hide items in the apple menu that don't exist in the wx menubar
     
-    wxMenuItem* appleItem = NULL;
-    wxMenuItem* wxItem = NULL;
+    wxMenuItem* appleItem = nullptr;
+    wxMenuItem* wxItem = nullptr;
 
     int menuid = wxApp::s_macAboutMenuItemId;
     appleItem = m_appleMenu->FindItem(menuid);
     wxItem = FindItem(menuid);
-    if ( appleItem != NULL )
+    if ( appleItem != nullptr )
     {
-        if ( wxItem == NULL )
+        if ( wxItem == nullptr )
             appleItem->GetPeer()->Hide();
         else 
             appleItem->SetItemLabel(wxItem->GetItemLabel());
@@ -635,9 +635,9 @@ void wxMenuBar::MacInstallMenuBar()
     menuid = wxApp::s_macPreferencesMenuItemId;
     appleItem = m_appleMenu->FindItem(menuid);
     wxItem = FindItem(menuid);
-    if ( appleItem != NULL )
+    if ( appleItem != nullptr )
     {
-        if ( wxItem == NULL )
+        if ( wxItem == nullptr )
             appleItem->GetPeer()->Hide();
         else 
             appleItem->SetItemLabel(wxItem->GetItemLabel());
@@ -715,7 +715,7 @@ wxMenu *wxMenuBar::Replace(size_t pos, wxMenu *menu, const wxString& title)
 {
     wxMenu *menuOld = wxMenuBarBase::Replace(pos, menu, title);
     if ( !menuOld )
-        return NULL;
+        return nullptr;
 
     wxMenuItem* item = m_rootMenu->FindItemByPosition(pos+firstMenuPos);
     m_rootMenu->Remove(item);
@@ -739,7 +739,7 @@ wxMenu *wxMenuBar::Remove(size_t pos)
 {
     wxMenu *menu = wxMenuBarBase::Remove(pos);
     if ( !menu )
-        return NULL;
+        return nullptr;
 
     wxMenuItem* item = m_rootMenu->FindItemByPosition(pos+firstMenuPos);
     m_rootMenu->Remove(item);
@@ -749,7 +749,7 @@ wxMenu *wxMenuBar::Remove(size_t pos)
 
 bool wxMenuBar::Append(wxMenu *menu, const wxString& title)
 {
-    WXHMENU submenu = menu ? menu->GetHMenu() : 0;
+    WXHMENU submenu = menu ? menu->GetHMenu() : nullptr;
         wxCHECK_MSG( submenu, false, wxT("can't append invalid menu to menubar") );
 
     if ( !wxMenuBarBase::Append(menu, title) )
